﻿using System;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using DayOf1440.NativeMethods;

namespace DayOf1440.Utils
{
    class ProcessHelper
    {

        private const int ErrorInsufficientBuffer = 0x7A;

        // ReSharper disable once InconsistentNaming
        public static void PerformAggressiveGC()
        {
            // delay some time to make sure that all windows are closed
            Task.Delay(2000).ContinueWith(t => GC.Collect(GC.MaxGeneration));
        }

        public static bool IsRunningAsUWP()
        {
            try
            {
                uint len = 0;
                var r = Kernel32.GetCurrentPackageFullName(ref len, null);

                return r == ErrorInsufficientBuffer;
            }
            catch (EntryPointNotFoundException)
            {
                return false;
            }
        }

        [SuppressMessage("ReSharper", "InconsistentNaming")]
        public static bool IsOnWindows10S()
        {
            const uint PRODUCT_CLOUD = 0x000000B2; // Windows 10 S
            const uint PRODUCT_CLOUDN = 0x000000B3; // Windows 10 S N

            Kernel32.GetProductInfo(Environment.OSVersion.Version.Major,
                Environment.OSVersion.Version.Minor, 0, 0, out var osType);

            return osType == PRODUCT_CLOUD || osType == PRODUCT_CLOUDN;
        }

        public static void WriteLog(string msg)
        {
            Debug.WriteLine(msg);

            var logFilePath = Path.Combine(SettingHelper.LocalDataPath, @"QuickLook.Exception.log");

            using (var writer = new StreamWriter(new FileStream(logFilePath, FileMode.OpenOrCreate,
                FileAccess.ReadWrite, FileShare.Read)))
            {
                writer.BaseStream.Seek(0, SeekOrigin.End);

                writer.WriteLine($"========{DateTime.Now}========");
                writer.WriteLine(msg);
                writer.WriteLine();
            }
        }
    }
}
